//+------------------------------------------------------------------+
//|                                                      COrders.mqh |
//|                                    Copyright 2017, Erwin Beckers |
//|                                      https://www.erwinbeckers.nl |
//+------------------------------------------------------------------+
#property copyright "Copyright 2017, Erwin Beckers"
#property link      "https://www.erwinbeckers.nl"
#property strict

#include <Erwin\COrder.mqh>

extern string  __Orders__           = "----- Money management settings------";;
extern bool    UseMoneyManagement   = false;               
extern double  RiskPercentage       = 2;                
extern double  FixedLotSize         = 0.01;  
extern int     MagicNumberBuy       = 1122;
extern int     MagicNumberSell      = 2211;
extern string  BuyComment           = "buy";
extern string  SellComment          = "sell";

class COrders
{
private:
   int     _today;
   int     _prevOrdersTotal;
   
public:
   int     TotalCount;              // # of open orders for this chart / ea
   int     BuyOrders;               // # of open buy orders for this chart / ea
   int     SellOrders;              // # of open sell orders for this chart / ea
   
   double  TotalProfit;             // total profit of all open orders for this chart / ea
   double  TotalProfitOfSellOrders; // total profit of all sell open orders for this chart / ea
   double  TotalProfitOfBuyOrders;  // total profit of all buy open orders for this chart / ea
   
   int     OrdersWon;               // Total # of orders won by this ea on this chart            
   int     OrdersLost;              // Total # of orders won by this ea on this chart       
   double  WinPercentage;           // percentage of orders won;     
   double  AllTimeProfit;           // all profit made by this ea on this chart            
   double  ProfitToday;             // profit made by this ea on this chart today
   double  ProfitYesterday;         // profit made by this ea on this chart yesterday
   double  ProfitFactor;            // profit factor
   double  ExpectedPayOff;          // expected payoff / trade   
   double  TotalLotsTraded;         // total nr of lots traded
   int     TotalTrades;             // total nr of trades traded
   double  AverageLotsPerTrade;     // average lot size / trade   
   int     GlobalOrderCount;        // # open orders for this account
   double  GlobalProfit;            // total profit of all open orders on this account
   
   double  HighestBuyPrice;
   double  LowestBuyPrice;
   double  HighestSellPrice;
   double  LowestSellPrice;
   bool    _filterCurrentSymbol;
   
   COrders(bool filterCurrentSymbol = true)
   {  
      _filterCurrentSymbol=filterCurrentSymbol;
      _prevOrdersTotal = 0;
      
      Refresh();
   }
   
   //------------------------------------------------------------------------------------
   //  GetOrderCount for symbol
   //------------------------------------------------------------------------------------
   int GetOrderCount(string symbol)
   {
      int cnt=0;
      for (int i=0; i < OrdersTotal();++i)
      {
         if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
         {
            if (OrderSymbol() == symbol || symbol==NULL)
            {
               if (OrderMagicNumber() == MagicNumberBuy || OrderMagicNumber() == MagicNumberSell) cnt++;
            }
         }
      }
      return cnt;
   }

   //------------------------------------------------------------------------------------
   //  Returns if order type is a buy or a sell
   //------------------------------------------------------------------------------------
   bool IsBuy(int orderType)
   {
      return (orderType  == OP_BUY ||  orderType== OP_BUYLIMIT || orderType== OP_BUYSTOP);
   }
    
   //------------------------------------------------------------------------------------
   //  Get the order with the highest opening price 
   //------------------------------------------------------------------------------------
   COrder* GetOrderWithHighestOpenPrice(bool buyOrders, bool sellOrders)
   {   
      double   highestPrice  =  0;
      int      ticket        = -1;
      double   profit        =  0;
      datetime openTime      =  0;
      
      for (int i=0; i < OrdersTotal(); i++)
      {
         if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
         {
            if (OrderSymbol() == Symbol() || !_filterCurrentSymbol)
            {
               if (OrderMagicNumber() == MagicNumberBuy || OrderMagicNumber() == MagicNumberSell)
               {
                  int orderType = OrderType();
                  bool isBuy  = IsBuy(orderType);
                  bool isSell = !IsBuy(orderType);
                  if ( (isBuy && buyOrders) || (isSell && sellOrders))
                  {
                     if (OrderOpenPrice() > highestPrice)
                     {
                        ticket        = OrderTicket();
                        highestPrice  = OrderOpenPrice();
                        profit        = OrderProfit() + OrderSwap() + OrderCommission();
                        openTime      = OrderOpenTime();
                     }
                  }
               }
            }
          }
       }
       if ( ticket < 0) return NULL;
       return new COrder(ticket, highestPrice,0 , profit, openTime, 0);
   }
    
   //------------------------------------------------------------------------------------
   //  Get the order with the lowest opening price 
   //------------------------------------------------------------------------------------
   COrder* GetOrderWithLowestOpenPrice(bool buyOrders, bool sellOrders)
   {   
      bool     first         = true;
      double   lowestPrice   =  0;
      int      ticket        = -1;
      double   profit        =  0;
      datetime openTime      =  0;
      for (int i=0; i < OrdersTotal(); i++)
      {
         if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
         {
            if (OrderSymbol() == Symbol() || !_filterCurrentSymbol)
            {
               if (OrderMagicNumber() == MagicNumberBuy || OrderMagicNumber() == MagicNumberSell)
               {
                  int orderType = OrderType();
                  bool isBuy  = IsBuy(orderType);
                  bool isSell = !IsBuy(orderType);
                  if ( (isBuy && buyOrders) || (isSell && sellOrders))
                  {
                     if (OrderOpenPrice() < lowestPrice || first)
                     {
                        first = false;
                        ticket        = OrderTicket();
                        lowestPrice   = OrderOpenPrice();
                        profit        = OrderProfit() + OrderSwap() + OrderCommission();
                        openTime      = OrderOpenTime();
                     }
                  }
               }
            }
          }
       }
       if ( ticket < 0) return NULL;
       return new COrder(ticket, lowestPrice, 0, profit, openTime, 0);
   }
    
    //------------------------------------------------------------------------------------
    //  Close order by ticket
    //------------------------------------------------------------------------------------
    bool CloseOrderByTicket(int ticket)
    {
      bool success = false;
      if(OrderSelect(ticket, SELECT_BY_TICKET, MODE_TRADES))
      {
         RefreshRates();
         double price   = MarketInfo(OrderSymbol(), MODE_ASK);
         int orderType  = OrderType();
         if (IsBuy(orderType)) price = MarketInfo(OrderSymbol(), MODE_BID);
         
         Print("Closing order ", OrderSymbol(),"  #",OrderTicket(), " lots:", OrderLots(), " price", price);
         success = OrderClose(OrderTicket(), OrderLots(), price, 3, Red);
         if (success)
         {
            if(OrderSelect(ticket, SELECT_BY_TICKET, MODE_HISTORY))
            {
            }
            else Print("------ UNABLE TO FIND ORDER IN HISTORY AFTER CLOSE --------");
         }
         else Print("------ UNABLE TO CLOSE ORDER --------");
      }
      else Print("------ UNABLE TO FIND ORDER TO CLOSE --------");
      Refresh();
      RefreshHistory();
      return success;
    }
    
    //------------------------------------------------------------------------------------
    //  Close order by ticket
    //------------------------------------------------------------------------------------
    void CloseOrderByType(string symbol, int orderType)
    {
      bool orderClosed;
      int  cnt=0;
      do
      {
         orderClosed=false;
         for (int i=0; i < OrdersTotal(); i++)
         {
            if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
            {
               if (OrderSymbol() == symbol && orderType == OrderType() )
               {
                  if (OrderMagicNumber() == MagicNumberBuy || OrderMagicNumber() == MagicNumberSell)
                  {
                     double price = MarketInfo(OrderSymbol(), MODE_ASK);
                     if (IsBuy(orderType))  price = MarketInfo(OrderSymbol(), MODE_BID);
                     
                     Print("Closing order ", OrderSymbol(),"  #",OrderTicket(), " lots:", OrderLots(), " price", price);
                     bool success = OrderClose(OrderTicket(), OrderLots(), price, 3, Red);
                     orderClosed=true;
                     cnt++;
                     break;
                  }
               }
             }
          }
       } while (orderClosed);
       if (cnt>0)
       {
          Refresh();
          RefreshHistory();
       }
    }
    
    //------------------------------------------------------------------------------------
    //  Close order by ticket
    //------------------------------------------------------------------------------------
    bool CloseOrder(COrder* order)
    {
      if (order.Ticket < 0) return false;
      bool success = false;
      if(OrderSelect(order.Ticket, SELECT_BY_TICKET, MODE_TRADES))
      {
         RefreshRates();
         double price   = MarketInfo(OrderSymbol(), MODE_ASK);
         int orderType  = OrderType();
         if (IsBuy(orderType)) price = MarketInfo(OrderSymbol(), MODE_BID);
         
         Print("Closing order ", OrderSymbol(),"  #",OrderTicket(), " lots:", OrderLots(), " price", price);
         success = OrderClose(OrderTicket(), OrderLots(), price, 3, Red);
         if (success)
         {
            if(OrderSelect(order.Ticket, SELECT_BY_TICKET, MODE_HISTORY))
            {
               order.Profit = OrderProfit() + OrderSwap() + OrderCommission();
              // Print("Order ",order.Ticket,"  closed with profit ",order.Profit);
            }
            else Print("------ UNABLE TO FIND ORDER IN HISTORY AFTER CLOSE --------");
         }
         else Print("------ UNABLE TO CLOSE ORDER --------");
      }
      else Print("------ UNABLE TO FIND ORDER TO CLOSE --------");
      Refresh();
      RefreshHistory();
      return success;
    }
    
   //------------------------------------------------------------------------------------
   //  CloseOrdersInProfit
   //------------------------------------------------------------------------------------
    bool CloseOrdersInProfit(string symbol )
    {
      bool orderClosed;
      do
      {
         orderClosed=false;
         for (int i=0; i < OrdersTotal(); i++)
         {
            if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
            {
               if ( OrderSymbol() == symbol)
               {
                  if (OrderMagicNumber() == MagicNumberBuy || OrderMagicNumber() == MagicNumberSell)
                  {
                     RefreshRates();
                     double orderProfit = OrderProfit() + OrderSwap() + OrderCommission();
                     if (orderProfit > 0)
                     {
                        double price = MarketInfo(OrderSymbol(), MODE_ASK);
                        int orderType = OrderType();
                        if (IsBuy(orderType))  price = MarketInfo(OrderSymbol(), MODE_BID);
                        
                        Print("Closing order ", OrderSymbol(),"  #",OrderTicket(), " lots:", OrderLots(), " price", price);
                        bool success = OrderClose(OrderTicket(), OrderLots(), price, 3, Red);
                        orderClosed=true;
                        break;
                     }
                  }
               }
            }
          }
       } while (orderClosed);
       Refresh();
       RefreshHistory();
       return orderClosed;
    }
    
   //------------------------------------------------------------------------------------
   //  CloseAllOrders
   //------------------------------------------------------------------------------------
    void CloseAllOrders()
    {
      Print("Closing all orders: total:",OrdersTotal());
      bool closedOrder=false;
      do {
         closedOrder=false;
         for (int i=0; i < OrdersTotal(); i++)
         {
            if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
            {
               if (OrderSymbol() == Symbol() || !_filterCurrentSymbol)
               {
                  if (OrderMagicNumber() == MagicNumberBuy || OrderMagicNumber() == MagicNumberSell)
                  {
                     closedOrder = true;
                     RefreshRates();
                     double price   = MarketInfo(OrderSymbol(), MODE_ASK);
                     int orderType  = OrderType();
                     if (IsBuy(orderType)) price = MarketInfo(OrderSymbol(), MODE_BID);
                     
                     Print("Closing order ", OrderSymbol(),"  #",OrderTicket(), " lots:", OrderLots(), " price", price);
                     bool success = OrderClose(OrderTicket(), OrderLots(), price, 3, Red);
                  }
               }
            }
          }
      } while (closedOrder);
      
      Refresh();
      RefreshHistory();
    }
    
    //------------------------------------------------------------------------------------
    //  Open a new buy order
    //------------------------------------------------------------------------------------
    int OpenBuyOrder(string symbol, double lotSize = 0, double sl=0, double tp=0)
    {
      if (lotSize <= 0) lotSize = GetLotSize(symbol);
      RefreshRates();
      double price  = MarketInfo(symbol, MODE_ASK);
      Print("Open buy order for ", symbol," lots:", lotSize, " price:", price,"  sl:",sl, " tp:",tp);
      int ticket    = OrderSend(symbol, OP_BUY, lotSize, price, 3, sl, tp, BuyComment, MagicNumberBuy);
      if (ticket < 0)
      {
         int ErrNumber = GetLastError();
         Print("  buy order failed err# ",ErrNumber, " symbol:", symbol, " lots:", lotSize, " price:", price, " sl:", sl, "  tp:",tp);
      }
      Refresh();
      return ticket;
    }
    
    //------------------------------------------------------------------------------------
    //  Open a new sell order
    //------------------------------------------------------------------------------------
    int OpenSellOrder(string symbol, double lotSize = 0, double sl=0, double tp=0)
    {
      if (lotSize <= 0) lotSize = GetLotSize(symbol);
      RefreshRates();
      double price  = MarketInfo(symbol, MODE_BID);
      Print("Open sell order for ", symbol," lots:", lotSize, " price:", price,"  sl:",sl, " tp:",tp);
      int ticket    = OrderSend(symbol, OP_SELL, lotSize, price, 3, sl,tp, SellComment, MagicNumberSell);
      if (ticket < 0)
      {
         int ErrNumber = GetLastError();
         Print("  sell order failed err# ",ErrNumber, " symbol:", symbol, " lots:", lotSize, " price:", price, " sl:", sl, "  tp:",tp);
      }
      Refresh();
      return ticket;
    }
    
   //------------------------------------------------------------------------------------
   //  change the take profit levels for all open orders of the order type specified
   //------------------------------------------------------------------------------------
    void SetTakeProfitLevel(int orderType, double takeProfit)
    {
      for (int i=0; i < OrdersTotal(); i++)
      {
         if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
         {
            if (OrderSymbol() == Symbol() || !_filterCurrentSymbol)
            {
               if (OrderMagicNumber() == MagicNumberBuy || OrderMagicNumber() == MagicNumberSell)
               {
                  if (orderType == OrderType() ) 
                  {
                     bool success = OrderModify(OrderTicket(), OrderOpenPrice(), OrderStopLoss(), takeProfit,0,clrNONE);
                  }
               }
            }
         }
       }
    }
    
   //------------------------------------------------------------------+
   // Generic Money management code
   //------------------------------------------------------------------+
   double GetLotSize(string symbol)
   {
      double leverage    = AccountLeverage();
      double minlot      = MarketInfo(symbol, MODE_MINLOT);
      double maxlot      = MarketInfo(symbol, MODE_MAXLOT);
      double lotsize     = MarketInfo(symbol, MODE_LOTSIZE);
      double stoplevel   = MarketInfo(symbol, MODE_STOPLEVEL);
      double askPrice    = MarketInfo(symbol, MODE_ASK);
      double lotStep     = MarketInfo(Symbol(),MODE_LOTSTEP) ;
      double lots        = FixedLotSize;
      int    lotDigits   = 2;
      if (lotStep      < 0.01) lotDigits = 3;
      else if (lotStep <  0.1) lotDigits = 2;
      else if (lotStep <  1.0) lotDigits = 1;
      else lotDigits = 0;
      
      if(UseMoneyManagement)
      {
         double amountToRisk = AccountFreeMargin() * RiskPercentage / 100.0;
         lots = NormalizeDouble( amountToRisk / 1000.0, lotDigits);
         if(lots < minlot) lots = minlot;
         if (lots > maxlot) lots = maxlot;
         if (AccountFreeMargin() < askPrice * lots * lotsize / leverage) 
         {
            Print("We have no money. Lots = ", lots, " , Free Margin = ", AccountFreeMargin());
            Comment("We have no money. Lots = ", lots, " , Free Margin = ", AccountFreeMargin());
         }
      }
      else
      {
         lots = NormalizeDouble(FixedLotSize, lotDigits);
      }
      return(lots);
   }

   //------------------------------------------------------------------------------------
   //  Get last closed order for a symbol
   //------------------------------------------------------------------------------------
   COrder* GetLastOrder(string symbol)
   {
      bool     first         = true;
      double   openPrice     =  0;
      double   closePrice    =  0;
      int      ticket        = -1;
      double   profit        =  0;
      datetime openTime      =  0;
      datetime closeTime     =  0;
      
      for (int i=0; i < OrdersHistoryTotal(); ++i)
      {
         if (OrderSelect(i, SELECT_BY_POS, MODE_HISTORY))
         {
            if (OrderMagicNumber() == MagicNumberBuy || OrderMagicNumber() == MagicNumberSell)
            {
               if (OrderSymbol() == symbol)
               {
                  if (ticket == -1 || OrderOpenTime() > openTime)
                  {
                     ticket    = OrderTicket();
                     openPrice = OrderOpenPrice();
                     closePrice= OrderClosePrice();
                     profit    = OrderProfit() + OrderSwap() + OrderCommission();
                     openTime  = OrderOpenTime();
                     closeTime = OrderCloseTime();
                  }
               }
            }
         }
      }
       if ( ticket < 0) return NULL;
       return new COrder(ticket, openPrice, closePrice, profit, openTime, closeTime);
   }

   //------------------------------------------------------------------------------------
   //  Refresh statistics of all closed trades
   //------------------------------------------------------------------------------------
    void RefreshHistory()
    {      
      // get history
      AllTimeProfit              = 0;
      OrdersWon                  = 0;
      OrdersLost                 = 0;     
      ProfitToday                = 0;
      ProfitYesterday            = 0; 
      WinPercentage              = 0;
      ProfitFactor               = 0;
      ExpectedPayOff             = 0;
      TotalLotsTraded            = 0;
      AverageLotsPerTrade        = 0;
      TotalTrades                = 0;
      double totalAmountWon      = 0;
      double totalAmountLost     = 0; 
      datetime now               = TimeCurrent();
      datetime yesterday         = TimeCurrent() - 60 * 60 * 24;
      for (int i=0; i < OrdersHistoryTotal(); ++i)
      {
        if(OrderSelect(i, SELECT_BY_POS,MODE_HISTORY))
        {
            if (OrderSymbol() == Symbol() || !_filterCurrentSymbol)
            {
               if (OrderMagicNumber() == MagicNumberBuy || OrderMagicNumber() == MagicNumberSell)
               {
                  TotalTrades = TotalTrades+1;
                  TotalLotsTraded += OrderLots();
                  double orderProfit = OrderProfit() + OrderSwap() + OrderCommission();
                  AllTimeProfit += orderProfit;
                  
                  if (orderProfit > 0) 
                  {
                     OrdersWon++;
                     totalAmountWon += orderProfit;
                  }
                  if (orderProfit < 0) 
                  {
                     OrdersLost++;
                     totalAmountLost += orderProfit;
                  }
                  
                  datetime orderTime = OrderCloseTime();
                  
                  if ( TimeDay(orderTime)   == TimeDay(now) &&
                       TimeMonth(orderTime) == TimeMonth(now) &&
                       TimeYear(orderTime)  == TimeYear(now) )
                  { 
                     ProfitToday += orderProfit;
                  }
                  
                  if ( TimeDay(orderTime)   == TimeDay(yesterday) &&
                       TimeMonth(orderTime) == TimeMonth(yesterday) &&
                       TimeYear(orderTime)  == TimeYear(yesterday) )
                  { 
                     ProfitYesterday += orderProfit;
                  }
               }
            }
        }
      }
      if (OrdersWon + OrdersLost != 0)
      {
         double won =(double)OrdersWon;
         double lost=(double)OrdersLost;
         WinPercentage =  (won / (won + lost) ) * 100.0;
      }
      
      if (totalAmountLost !=0)
      {
         ProfitFactor = MathAbs( totalAmountWon / totalAmountLost);
      }
      
      if (TotalTrades > 0)
      {
         double dTrades=TotalTrades;
         ExpectedPayOff = AllTimeProfit / dTrades;
         AverageLotsPerTrade = TotalLotsTraded /  dTrades;
      }
   }
   
   
   //------------------------------------------------------------------------------------
   //  Refresh statistics of all open trades
   //------------------------------------------------------------------------------------
   void Refresh()
   {
      // refresh history when new day arrives
      if ( (TimeDay( TimeCurrent()) != _today) || ( OrdersTotal() != _prevOrdersTotal))
      {
        _today = TimeDay( TimeCurrent());
        _prevOrdersTotal = OrdersTotal();
        RefreshHistory(); 
      }
   
      GlobalOrderCount = 0;
      GlobalProfit     = 0;
      TotalCount = 0;
      SellOrders = 0;
      BuyOrders  = 0;
      TotalProfit= 0;
      TotalProfitOfSellOrders = 0;
      TotalProfitOfBuyOrders  = 0;
      HighestBuyPrice  = 0;
      LowestBuyPrice   = 999999;
      HighestSellPrice = 0;
      LowestSellPrice  = 999999;
      
      for (int i=0; i < OrdersTotal(); ++i)
      {
         if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
         {
            double orderProfit = OrderProfit() + OrderSwap() + OrderCommission();
            GlobalOrderCount++;
            GlobalProfit += orderProfit;
            if (OrderSymbol() == Symbol() || !_filterCurrentSymbol)
            {
               double price = OrderOpenPrice();
               if (OrderMagicNumber() == MagicNumberBuy || OrderMagicNumber() == MagicNumberSell)
               {
                  TotalProfit += orderProfit;
                  TotalCount++;
                  
                  int orderType=OrderType();
                  if (IsBuy(orderType)) 
                  {
                     BuyOrders++;
                     TotalProfitOfBuyOrders += orderProfit;
                     HighestBuyPrice = MathMax(HighestBuyPrice, price);
                     LowestBuyPrice  = MathMin(LowestBuyPrice, price);
                  }
                  else
                  {
                     SellOrders=SellOrders+1;
                     TotalProfitOfSellOrders += orderProfit;
                     HighestSellPrice = MathMax(HighestSellPrice, price);
                     LowestSellPrice  = MathMin(LowestSellPrice, price);
                  }
               }
            }
         }
      }
   }
};